@@ -41,6 +41,7 @@
 #include "bamf-application.h"
 #include "bamf-application-private.h"
 #include "bamf-indicator.h"
+#include "bamf-tab.h"
 
 #include <dbus/dbus.h>
 #include <dbus/dbus-glib.h>
@@ -95,7 +96,7 @@ on_view_closed (BamfView *view, BamfFactory *self)
   
   g_return_if_fail (BAMF_IS_VIEW (view));
   
-  path = bamf_view_get_path (view);
+  path = _bamf_view_get_path (view);
   if (path)
     g_hash_table_remove (self->priv->views, path);
   
@@ -128,7 +129,7 @@ bamf_factory_register_view (BamfFactory *self, BamfView *view, const char *path)
 }
 
 BamfApplication * 
-bamf_factory_app_for_file (BamfFactory * factory,
+_bamf_factory_app_for_file (BamfFactory * factory,
                            const char * path,
                            gboolean create)
 {
@@ -184,30 +185,34 @@ BamfFactoryViewType compute_factory_type_by_str (const char *type)
         {
           factory_type = BAMF_FACTORY_VIEW;
         }
+      else if (g_strcmp0 (type, "tab") == 0)
+        {
+          factory_type = BAMF_FACTORY_TAB;
+        }
     }
 
   return factory_type;
 }
 
 BamfView * 
-bamf_factory_view_for_path (BamfFactory * factory, const char * path)
+_bamf_factory_view_for_path (BamfFactory * factory, const char * path)
 {
-  return bamf_factory_view_for_path_type (factory, path, BAMF_FACTORY_NONE);
+  return _bamf_factory_view_for_path_type (factory, path, BAMF_FACTORY_NONE);
 }
 
 BamfView * 
-bamf_factory_view_for_path_type_str (BamfFactory * factory, const char * path,
+_bamf_factory_view_for_path_type_str (BamfFactory * factory, const char * path,
                                                             const char * type)
 {
   g_return_val_if_fail (BAMF_IS_FACTORY (factory), NULL);
   BamfFactoryViewType factory_type = compute_factory_type_by_str (type);
 
-  return bamf_factory_view_for_path_type (factory, path, factory_type);
+  return _bamf_factory_view_for_path_type (factory, path, factory_type);
 }
 
 BamfView *
-bamf_factory_view_for_path_type (BamfFactory * factory, const char * path, 
-                                 BamfFactoryViewType type)
+_bamf_factory_view_for_path_type (BamfFactory * factory, const char * path,
+                                                         BamfFactoryViewType type)
 {
   GHashTable *views;
   BamfView *view;
@@ -228,7 +233,7 @@ bamf_factory_view_for_path_type (BamfFactory * factory, const char * path,
   if (type == BAMF_FACTORY_NONE)
     {
       view = g_object_new (BAMF_TYPE_VIEW, NULL);
-      bamf_view_set_path (view, path);
+      _bamf_view_set_path (view, path);
       type = compute_factory_type_by_str (bamf_view_get_view_type (view));
       g_object_unref (view);
       view = NULL;
@@ -248,8 +253,12 @@ bamf_factory_view_for_path_type (BamfFactory * factory, const char * path,
     case BAMF_FACTORY_INDICATOR:
       view = BAMF_VIEW (bamf_indicator_new (path));
       break;
+    case BAMF_FACTORY_TAB:
+      view = BAMF_VIEW (bamf_tab_new (path));
+      break;
     case BAMF_FACTORY_NONE:
       view = NULL;
+      break;
   }
 
   created = TRUE;
@@ -280,10 +289,10 @@ bamf_factory_view_for_path_type (BamfFactory * factory, const char * path,
 
           /* If the primary search doesn't give out any result, we fallback
            * to children window comparison */
-          if (!list_desktop_file && !matched_view)
+          if (!matched_view)
             {
               GList *list_children, *ll;
-              list_children = bamf_application_get_cached_xids (list_app);
+              list_children = _bamf_application_get_cached_xids (list_app);
 
               for (ll = local_children; ll; ll = ll->next)
                 {
@@ -332,7 +341,7 @@ bamf_factory_view_for_path_type (BamfFactory * factory, const char * path,
       g_object_unref (view);
 
       view = matched_view;
-      bamf_view_set_path (view, path);
+      _bamf_view_set_path (view, path);
       g_object_ref_sink (view);
     }
 
@@ -351,7 +360,7 @@ bamf_factory_view_for_path_type (BamfFactory * factory, const char * path,
 }
 
 BamfFactory * 
-bamf_factory_get_default (void)
+_bamf_factory_get_default (void)
 {
   
   if (BAMF_IS_FACTORY (factory))
