@@ -39,6 +39,7 @@
 #include "bamf-factory.h"
 #include "bamf-window.h"
 #include "bamf-marshal.h"
+#include "bamf-tab.h"
 
 #include <dbus/dbus.h>
 #include <dbus/dbus-glib.h>
@@ -67,7 +68,7 @@ enum
   URGENT_CHANGED,
   VISIBLE_CHANGED,
   NAME_CHANGED,
-
+  CHILD_MOVED,
   LAST_SIGNAL,
 };
 
@@ -142,6 +143,15 @@ bamf_view_get_flag (BamfView *view, guint flag)
   return priv->set_flags & flag;
 }
 
+/**
+ * bamf_view_get_children:
+ * @view: a #BamfView
+ *
+ * Note: Makes sever dbus calls the first time this is called on a view. Dbus messaging is reduced afterwards.
+ *
+ * Returns: (element-type Bamf.View) (transfer container): Returns a list of #BamfView which must be
+ *           freed after usage. Elements of the list are owned by bamf and should not be unreffed.
+ */
 GList *
 bamf_view_get_children (BamfView *view)
 {
@@ -156,7 +166,7 @@ bamf_view_get_children (BamfView *view)
   if (BAMF_VIEW_GET_CLASS (view)->get_children)
     return BAMF_VIEW_GET_CLASS (view)->get_children (view);
 
-  if (!bamf_view_remote_ready (view))
+  if (!_bamf_view_remote_ready (view))
     return NULL;
 
   priv = view->priv;
@@ -183,7 +193,7 @@ bamf_view_get_children (BamfView *view)
 
   for (i = len-1; i >= 0; i--)
     {
-      view = bamf_factory_view_for_path (bamf_factory_get_default (), children[i]);
+      view = _bamf_factory_view_for_path (_bamf_factory_get_default (), children[i]);
       results = g_list_prepend (results, g_object_ref (view));
     }
 
@@ -204,7 +214,7 @@ bamf_view_get_boolean (BamfView *self, const char *method_name, guint flag)
   if (bamf_view_flag_is_set (self, flag))
     return bamf_view_get_flag (self, flag);
 
-  if (!bamf_view_remote_ready (self))
+  if (!_bamf_view_remote_ready (self))
     return FALSE;
 
   if (!dbus_g_proxy_call (priv->proxy,
@@ -224,6 +234,12 @@ bamf_view_get_boolean (BamfView *self, const char *method_name, guint flag)
   return result;
 }
 
+/**
+ * bamf_view_is_closed:
+ * @view: a #BamfView
+ *
+ * Determines if the view is closed or not.
+ */
 gboolean
 bamf_view_is_closed (BamfView *view)
 {
@@ -232,6 +248,12 @@ bamf_view_is_closed (BamfView *view)
   return view->priv->is_closed;
 }
 
+/**
+ * bamf_view_is_active:
+ * @view: a #BamfView
+ *
+ * Determines if the view is currently active and focused by the user. Useful for an active window indicator. 
+ */
 gboolean
 bamf_view_is_active (BamfView *view)
 {
@@ -244,7 +266,14 @@ bamf_view_is_active (BamfView *view)
 
 }
 
-gboolean          
+/**
+ * bamf_view_user_visible:
+ * @view: a #BamfView
+ *
+ * Returns a boolean useful for determining if a particular view is "user visible". User visible
+ * is a concept relating to whether or not a window should be shown in a launcher tasklist.
+ */
+gboolean
 bamf_view_user_visible (BamfView *self)
 {
   g_return_val_if_fail (BAMF_IS_VIEW (self), FALSE);
@@ -253,6 +282,12 @@ bamf_view_user_visible (BamfView *self)
 
 }
 
+/**
+ * bamf_view_is_running:
+ * @view: a #BamfView
+ *
+ * Determines if the view is currently running. Useful for a running window indicator. 
+ */
 gboolean
 bamf_view_is_running (BamfView *self)
 {
@@ -264,6 +299,12 @@ bamf_view_is_running (BamfView *self)
   return bamf_view_get_boolean (self, "IsRunning", BAMF_VIEW_RUNNING_FLAG);
 }
 
+/**
+ * bamf_view_is_urgent:
+ * @view: a #BamfView
+ *
+ * Determines if the view is currently requiring attention. Useful for a running window indicator. 
+ */
 gboolean
 bamf_view_is_urgent (BamfView *self)
 {
@@ -276,7 +317,7 @@ bamf_view_is_urgent (BamfView *self)
 }
 
 void
-bamf_view_set_name (BamfView *view, const char *name)
+_bamf_view_set_name (BamfView *view, const char *name)
 {
   g_return_if_fail (BAMF_IS_VIEW (view));
   
@@ -296,7 +337,7 @@ bamf_view_set_name (BamfView *view, const char *name)
 }
 
 void
-bamf_view_set_icon (BamfView *view, const char *icon)
+_bamf_view_set_icon (BamfView *view, const char *icon)
 {
   g_return_if_fail (BAMF_IS_VIEW (view));
 
@@ -336,6 +377,12 @@ bamf_view_set_sticky (BamfView *view, gboolean value)
     g_object_unref (view);
 }
 
+/**
+ * bamf_view_get_icon:
+ * @view: a #BamfView
+ *
+ * Gets the icon of a view. This icon is used to visually represent the view. 
+ */
 gchar *
 bamf_view_get_icon (BamfView *self)
 {
@@ -349,7 +396,7 @@ bamf_view_get_icon (BamfView *self)
   if (BAMF_VIEW_GET_CLASS (self)->get_icon)
     return BAMF_VIEW_GET_CLASS (self)->get_icon (self);
 
-  if (!bamf_view_remote_ready (self))
+  if (!_bamf_view_remote_ready (self))
     return g_strdup (priv->local_icon);
 
   if (!dbus_g_proxy_call (priv->proxy,
@@ -374,6 +421,12 @@ bamf_view_get_icon (BamfView *self)
   return icon;
 }
 
+/**
+ * bamf_view_get_name:
+ * @view: a #BamfView
+ *
+ * Gets the name of a view. This name is a short name best used to represent the view with text. 
+ */
 gchar *
 bamf_view_get_name (BamfView *self)
 {
@@ -387,7 +440,7 @@ bamf_view_get_name (BamfView *self)
   if (BAMF_VIEW_GET_CLASS (self)->get_name)
     return BAMF_VIEW_GET_CLASS (self)->get_name (self);
 
-  if (!bamf_view_remote_ready (self))
+  if (!_bamf_view_remote_ready (self))
     return g_strdup (priv->local_name);
     
   if (!dbus_g_proxy_call (priv->proxy,
@@ -413,7 +466,7 @@ bamf_view_get_name (BamfView *self)
 }
 
 gboolean 
-bamf_view_remote_ready (BamfView *view)
+_bamf_view_remote_ready (BamfView *view)
 {
   if (BAMF_IS_VIEW (view) && view->priv->proxy)
     return !view->priv->is_closed;
@@ -421,6 +474,15 @@ bamf_view_remote_ready (BamfView *view)
   return FALSE;
 }
 
+/**
+ * bamf_view_get_view_type:
+ * @view: a #BamfView
+ *
+ * The view type of a window is a short string used to represent all views of the same class. These
+ * descriptions should not be used to do casting as they are not considered stable.
+ *
+ * Virtual: view_type
+ */
 const gchar *
 bamf_view_get_view_type (BamfView *self)
 {
@@ -464,14 +526,29 @@ bamf_view_get_click_suggestion (BamfView *self)
   return BAMF_CLICK_BEHAVIOR_NONE;
 }
 
+static void
+bamf_view_child_xid_changed (GObject *object, GParamSpec *pspec, gpointer user_data)
+{
+  BamfView *self;
+  
+  self = (BamfView *)user_data;
+  
+  g_signal_emit (G_OBJECT (self), view_signals[CHILD_MOVED], 0, BAMF_VIEW (object));
+  g_signal_emit (G_OBJECT (self), view_signals[VISIBLE_CHANGED], 0);
+}
+
 static void
 bamf_view_on_child_added (DBusGProxy *proxy, char *path, BamfView *self)
 {
   BamfView *view;
   BamfViewPrivate *priv;
 
-  view = bamf_factory_view_for_path (bamf_factory_get_default (), path);
+  view = _bamf_factory_view_for_path (_bamf_factory_get_default (), path);
   priv = self->priv;
+  
+  if (BAMF_IS_TAB (view))
+    g_signal_connect (view, "notify::xid", 
+                      G_CALLBACK (bamf_view_child_xid_changed), self);
 
   if (priv->cached_children)
     {
@@ -487,8 +564,11 @@ bamf_view_on_child_removed (DBusGProxy *proxy, char *path, BamfView *self)
 {
   BamfView *view;
   BamfViewPrivate *priv;
-  view = bamf_factory_view_for_path (bamf_factory_get_default (), path);
+  view = _bamf_factory_view_for_path (_bamf_factory_get_default (), path);
   priv = self->priv;
+  
+  if (BAMF_IS_TAB (view))
+    g_signal_handlers_disconnect_by_func (view, bamf_view_on_child_added, self);
 
   if (priv->cached_children)
     {
@@ -646,43 +726,43 @@ bamf_view_unset_proxy (BamfView *self)
     return;
 
   dbus_g_proxy_disconnect_signal (priv->proxy,
-                                  "ActiveChanged",
-                                  (GCallback) bamf_view_on_active_changed,
-                                  self);
+                                      "ActiveChanged",
+                                      (GCallback) bamf_view_on_active_changed,
+                                      self);
 
-  dbus_g_proxy_disconnect_signal (priv->proxy,
-                                  "Closed",
-                                  (GCallback) bamf_view_on_closed,
-                                  self);
+      dbus_g_proxy_disconnect_signal (priv->proxy,
+                                      "Closed",
+                                      (GCallback) bamf_view_on_closed,
+                                      self);
 
-  dbus_g_proxy_disconnect_signal (priv->proxy,
-                                  "ChildAdded",
-                                  (GCallback) bamf_view_on_child_added,
-                                  self);
+      dbus_g_proxy_disconnect_signal (priv->proxy,
+                                      "ChildAdded",
+                                      (GCallback) bamf_view_on_child_added,
+                                      self);
 
-  dbus_g_proxy_disconnect_signal (priv->proxy,
-                                  "ChildRemoved",
-                                  (GCallback) bamf_view_on_child_removed,
-                                  self);
+      dbus_g_proxy_disconnect_signal (priv->proxy,
+                                      "ChildRemoved",
+                                      (GCallback) bamf_view_on_child_removed,
+                                      self);
 
-  dbus_g_proxy_disconnect_signal (priv->proxy,
-                                  "RunningChanged",
-                                  (GCallback) bamf_view_on_running_changed,
-                                  self);
+      dbus_g_proxy_disconnect_signal (priv->proxy,
+                                      "RunningChanged",
+                                      (GCallback) bamf_view_on_running_changed,
+                                      self);
 
-  dbus_g_proxy_disconnect_signal (priv->proxy,
-                                 "UrgentChanged",
-                                 (GCallback) bamf_view_on_urgent_changed,
-                                 self);
+      dbus_g_proxy_disconnect_signal (priv->proxy,
+                                     "UrgentChanged",
+                                     (GCallback) bamf_view_on_urgent_changed,
+                                     self);
+  
+      dbus_g_proxy_disconnect_signal (priv->proxy,
+                                     "UserVisibleChanged",
+                                     (GCallback) bamf_view_on_user_visible_changed,
+                                     self);
 
-  dbus_g_proxy_disconnect_signal (priv->proxy,
-                                 "UserVisibleChanged",
-                                 (GCallback) bamf_view_on_user_visible_changed,
-                                 self);
-
-  g_signal_handlers_disconnect_by_func (priv->proxy, on_view_proxy_destroyed, self);
-  g_object_unref (priv->proxy);
-  priv->proxy = NULL;
+      g_signal_handlers_disconnect_by_func (priv->proxy, on_view_proxy_destroyed, self);
+      g_object_unref (priv->proxy);
+      priv->proxy = NULL;
 }
 
 static void
@@ -731,7 +811,7 @@ bamf_view_dispose (GObject *object)
 }
 
 const char *
-bamf_view_get_path (BamfView *view)
+_bamf_view_get_path (BamfView *view)
 {
   g_return_val_if_fail (BAMF_IS_VIEW (view), NULL);
 
@@ -739,7 +819,7 @@ bamf_view_get_path (BamfView *view)
 }
 
 void
-bamf_view_reset_flags (BamfView *view)
+_bamf_view_reset_flags (BamfView *view)
 {
   BamfViewPrivate *priv;
   g_return_if_fail (BAMF_IS_VIEW (view));
@@ -773,7 +853,7 @@ bamf_view_reset_flags (BamfView *view)
 }
 
 void
-bamf_view_set_path (BamfView *view, const char *path)
+_bamf_view_set_path (BamfView *view, const char *path)
 {
   BamfViewPrivate *priv;
 
@@ -897,7 +977,7 @@ bamf_view_set_path (BamfView *view, const char *path)
 
   if (bamf_view_is_sticky (view))
     {
-      bamf_view_reset_flags (view);
+      _bamf_view_reset_flags (view);
     }
 
   if (BAMF_VIEW_GET_CLASS (view)->set_path)
@@ -940,82 +1020,92 @@ bamf_view_class_init (BamfViewClass *klass)
   g_type_class_add_private (obj_class, sizeof (BamfViewPrivate));
 
   view_signals [ACTIVE_CHANGED] = 
-  	g_signal_new ("active-changed",
-  	              G_OBJECT_CLASS_TYPE (klass),
-  	              G_SIGNAL_RUN_FIRST,
-  	              G_STRUCT_OFFSET (BamfViewClass, active_changed), 
-  	              NULL, NULL,
-  	              g_cclosure_marshal_VOID__BOOLEAN,
-  	              G_TYPE_NONE, 1, 
-  	              G_TYPE_BOOLEAN);
+        g_signal_new ("active-changed",
+                      G_OBJECT_CLASS_TYPE (klass),
+                      G_SIGNAL_RUN_FIRST,
+                      G_STRUCT_OFFSET (BamfViewClass, active_changed), 
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__BOOLEAN,
+                      G_TYPE_NONE, 1, 
+                      G_TYPE_BOOLEAN);
 
   view_signals [CLOSED] = 
-  	g_signal_new ("closed",
-  	              G_OBJECT_CLASS_TYPE (klass),
-  	              G_SIGNAL_RUN_FIRST,
-  	              G_STRUCT_OFFSET (BamfViewClass, closed), 
-  	              NULL, NULL,
-  	              g_cclosure_marshal_VOID__VOID,
-  	              G_TYPE_NONE, 0);
+        g_signal_new ("closed",
+                      G_OBJECT_CLASS_TYPE (klass),
+                      G_SIGNAL_RUN_FIRST,
+                      G_STRUCT_OFFSET (BamfViewClass, closed), 
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__VOID,
+                      G_TYPE_NONE, 0);
 
   view_signals [CHILD_ADDED] = 
-  	g_signal_new ("child-added",
-  	              G_OBJECT_CLASS_TYPE (klass),
-  	              G_SIGNAL_RUN_FIRST,
-  	              G_STRUCT_OFFSET (BamfViewClass, child_added), 
-  	              NULL, NULL,
-  	              g_cclosure_marshal_VOID__OBJECT,
-  	              G_TYPE_NONE, 1, 
-  	              BAMF_TYPE_VIEW);
+        g_signal_new ("child-added",
+                      G_OBJECT_CLASS_TYPE (klass),
+                      G_SIGNAL_RUN_FIRST,
+                      G_STRUCT_OFFSET (BamfViewClass, child_added), 
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__OBJECT,
+                      G_TYPE_NONE, 1, 
+                      BAMF_TYPE_VIEW);
 
   view_signals [CHILD_REMOVED] = 
-  	g_signal_new ("child-removed",
-  	              G_OBJECT_CLASS_TYPE (klass),
-  	              G_SIGNAL_RUN_FIRST,
-  	              G_STRUCT_OFFSET (BamfViewClass, child_removed), 
-  	              NULL, NULL,
-  	              g_cclosure_marshal_VOID__OBJECT,
-  	              G_TYPE_NONE, 1, 
-  	              BAMF_TYPE_VIEW);
+        g_signal_new ("child-removed",
+                      G_OBJECT_CLASS_TYPE (klass),
+                      G_SIGNAL_RUN_FIRST,
+                      G_STRUCT_OFFSET (BamfViewClass, child_removed), 
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__OBJECT,
+                      G_TYPE_NONE, 1, 
+                      BAMF_TYPE_VIEW);
+  
+  view_signals [CHILD_MOVED] = 
+    g_signal_new ("child-moved",
+                  G_OBJECT_CLASS_TYPE (klass),
+                  G_SIGNAL_RUN_FIRST,
+                  0,
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__OBJECT,
+                  G_TYPE_NONE, 1,
+                  BAMF_TYPE_VIEW);
 
   view_signals [RUNNING_CHANGED] = 
-  	g_signal_new ("running-changed",
-  	              G_OBJECT_CLASS_TYPE (klass),
-  	              G_SIGNAL_RUN_FIRST,
-  	              G_STRUCT_OFFSET (BamfViewClass, running_changed), 
-  	              NULL, NULL,
-  	              g_cclosure_marshal_VOID__BOOLEAN,
-  	              G_TYPE_NONE, 1, 
-  	              G_TYPE_BOOLEAN);
+        g_signal_new ("running-changed",
+                      G_OBJECT_CLASS_TYPE (klass),
+                      G_SIGNAL_RUN_FIRST,
+                      G_STRUCT_OFFSET (BamfViewClass, running_changed), 
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__BOOLEAN,
+                      G_TYPE_NONE, 1, 
+                      G_TYPE_BOOLEAN);
 
   view_signals [URGENT_CHANGED] = 
-  	g_signal_new ("urgent-changed",
-  	              G_OBJECT_CLASS_TYPE (klass),
-  	              G_SIGNAL_RUN_FIRST,
-  	              G_STRUCT_OFFSET (BamfViewClass, urgent_changed), 
-  	              NULL, NULL,
-  	              g_cclosure_marshal_VOID__BOOLEAN,
-  	              G_TYPE_NONE, 1, 
-  	              G_TYPE_BOOLEAN);
+        g_signal_new ("urgent-changed",
+                      G_OBJECT_CLASS_TYPE (klass),
+                      G_SIGNAL_RUN_FIRST,
+                      G_STRUCT_OFFSET (BamfViewClass, urgent_changed), 
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__BOOLEAN,
+                      G_TYPE_NONE, 1, 
+                      G_TYPE_BOOLEAN);
   
   view_signals [VISIBLE_CHANGED] = 
-  	g_signal_new ("user-visible-changed",
-  	              G_OBJECT_CLASS_TYPE (klass),
-  	              G_SIGNAL_RUN_FIRST,
-  	              G_STRUCT_OFFSET (BamfViewClass, user_visible_changed), 
-  	              NULL, NULL,
-  	              g_cclosure_marshal_VOID__BOOLEAN,
-  	              G_TYPE_NONE, 1, 
-  	              G_TYPE_BOOLEAN);
+        g_signal_new ("user-visible-changed",
+                      G_OBJECT_CLASS_TYPE (klass),
+                      G_SIGNAL_RUN_FIRST,
+                      G_STRUCT_OFFSET (BamfViewClass, user_visible_changed), 
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__BOOLEAN,
+                      G_TYPE_NONE, 1, 
+                      G_TYPE_BOOLEAN);
 
   view_signals [NAME_CHANGED] =
         g_signal_new ("name-changed",
                       G_OBJECT_CLASS_TYPE (klass),
                       0,
                       0, NULL, NULL,
-                      bamf_marshal_VOID__STRING_STRING,
-  	              G_TYPE_NONE, 2,
-  	              G_TYPE_STRING,
+                      _bamf_marshal_VOID__STRING_STRING,
+                      G_TYPE_NONE, 2,
+                      G_TYPE_STRING,
                       G_TYPE_STRING);
 }
 
