@@ -30,12 +30,12 @@
 #include <glib-object.h>
 #include <libwnck/libwnck.h>
 
-#define BAMF_TYPE_LEGACY_WINDOW			(bamf_legacy_window_get_type ())
-#define BAMF_LEGACY_WINDOW(obj)			(G_TYPE_CHECK_INSTANCE_CAST ((obj), BAMF_TYPE_LEGACY_WINDOW, BamfLegacyWindow))
-#define BAMF_IS_LEGACY_WINDOW(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), BAMF_TYPE_LEGACY_WINDOW))
-#define BAMF_LEGACY_WINDOW_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST ((klass), BAMF_TYPE_LEGACY_WINDOW, BamfLegacyWindowClass))
-#define BAMF_IS_LEGACY_WINDOW_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), BAMF_TYPE_LEGACY_WINDOW))
-#define BAMF_LEGACY_WINDOW_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS ((obj), BAMF_TYPE_LEGACY_WINDOW, BamfLegacyWindowClass))
+#define BAMF_TYPE_LEGACY_WINDOW                 (bamf_legacy_window_get_type ())
+#define BAMF_LEGACY_WINDOW(obj)                 (G_TYPE_CHECK_INSTANCE_CAST ((obj), BAMF_TYPE_LEGACY_WINDOW, BamfLegacyWindow))
+#define BAMF_IS_LEGACY_WINDOW(obj)              (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BAMF_TYPE_LEGACY_WINDOW))
+#define BAMF_LEGACY_WINDOW_CLASS(klass)         (G_TYPE_CHECK_CLASS_CAST ((klass), BAMF_TYPE_LEGACY_WINDOW, BamfLegacyWindowClass))
+#define BAMF_IS_LEGACY_WINDOW_CLASS(klass)      (G_TYPE_CHECK_CLASS_TYPE ((klass), BAMF_TYPE_LEGACY_WINDOW))
+#define BAMF_LEGACY_WINDOW_GET_CLASS(obj)       (G_TYPE_INSTANCE_GET_CLASS ((obj), BAMF_TYPE_LEGACY_WINDOW, BamfLegacyWindowClass))
 
 #define BAMF_LEGACY_WINDOW_SIGNAL_NAME_CHANGED     "name-changed"
 #define BAMF_LEGACY_WINDOW_SIGNAL_STATE_CHANGED    "state-changed"
@@ -77,8 +77,6 @@ struct _BamfLegacyWindowClass
   char       * (*get_app_id)              (BamfLegacyWindow *legacy_window);
   char       * (*get_unique_bus_name)     (BamfLegacyWindow *legacy_window);
   char       * (*get_menu_object_path)    (BamfLegacyWindow *legacy_window);
-  char       * (*get_hint)                (BamfLegacyWindow *legacy_window,
-                                           const gchar *name);
   gint         (*get_pid)                 (BamfLegacyWindow *legacy_window);
   guint32      (*get_xid)                 (BamfLegacyWindow *legacy_window);
   gboolean     (*needs_attention)         (BamfLegacyWindow *legacy_window);
@@ -90,9 +88,8 @@ struct _BamfLegacyWindowClass
   BamfWindowMaximizationType (*maximized) (BamfLegacyWindow *legacy_window);
   BamfWindowType (*get_window_type)       (BamfLegacyWindow *legacy_window);
   void         (*get_geometry)            (BamfLegacyWindow *legacy_window,
-                                           gint *x, gint *y, gint *w, gint *h);
-  void         (*set_hint)                (BamfLegacyWindow *legacy_window,
-                                           const gchar *name, const gchar *value);
+                                           gint *x, gint *y,
+                                           gint *width, gint *height);
   void         (*reopen)                  (BamfLegacyWindow *legacy_window);
 
   /*< signals >*/
@@ -144,12 +141,7 @@ char             * bamf_legacy_window_get_exec_string      (BamfLegacyWindow *se
 
 BamfLegacyWindow * bamf_legacy_window_get_transient        (BamfLegacyWindow *self);
 
-char             * bamf_legacy_window_get_hint             (BamfLegacyWindow *self,
-                                                            const char *name);
-
-void               bamf_legacy_window_set_hint             (BamfLegacyWindow *self,
-                                                            const char *name,
-                                                            const char *value);
+char             * bamf_legacy_window_get_utf8_xprop       (BamfLegacyWindow *self, const char* prop);
 
 gint               bamf_legacy_window_get_stacking_position (BamfLegacyWindow *self);
 
