@@ -25,6 +25,7 @@
 #include "bamf-application.h"
 #include "bamf-window.h"
 #include "bamf-legacy-window.h"
+#include "bamf-unity-webapps-observer.h"
 
 struct _BamfMatcherPrivate
 {
@@ -42,10 +43,11 @@ struct _BamfMatcherPrivate
   BamfView        * active_app;
   BamfView        * active_win;
   guint             dispatch_changes_id;
+  
+  BamfUnityWebappsObserver *webapps_observer;
 };
 
 BamfApplication * bamf_matcher_get_application_by_desktop_file (BamfMatcher *self, const char *desktop_file);
 BamfApplication * bamf_matcher_get_application_by_xid (BamfMatcher *self, guint xid);
-char * bamf_matcher_get_trimmed_exec (BamfMatcher *self, const char *exec);
 
 #endif
