@@ -26,6 +26,9 @@
 #include "bamf-window.h"
 #include "bamf-legacy-screen.h"
 #include "bamf-indicator-source.h"
+#include "bamf-xutils.h"
+
+#include "bamf-unity-webapps-application.h"
 #include <strings.h>
 
 G_DEFINE_TYPE (BamfMatcher, bamf_matcher, BAMF_DBUS_TYPE_MATCHER_SKELETON);
@@ -48,22 +51,6 @@ typedef enum
 static BamfMatcher *static_matcher;
 static guint matcher_signals[LAST_SIGNAL] = { 0 };
 
-// Prefixes to be ignored in exec strings
-const gchar* EXEC_BAD_PREFIXES[] =
-{
-  "^gksu(do)?$", "^sudo$", "^su-to-root$", "^amdxdg-su$", "^java(ws)?$",
-  "^mono$", "^ruby$", "^padsp$", "^aoss$", "^python(\\d.\\d)?$", "^(ba)?sh$",
-  "^perl$", "^env$", "^xdg-open$",
-  /* javaws strings: */ "^net\\.sourceforge\\.jnlp\\.runtime\\.Boot$", "^rt.jar$"
-};
-
-// Prefixes that must be considered starting point of exec strings
-const gchar* EXEC_GOOD_PREFIXES[] =
-{
-  "^gnome-control-center$", "^libreoffice$", "^ooffice$", "^wine$", "^steam$",
-  "^sol$"
-};
-
 static void
 on_view_active_changed (BamfView *view, gboolean active, BamfMatcher *matcher)
 {
@@ -227,8 +214,7 @@ emit_paths_changed (gpointer user_data)
   return FALSE;
 }
 
-static void
-bamf_matcher_prepare_path_change (BamfMatcher *self, const gchar *desktop_file, ViewChangeType change_type)
+static void bamf_matcher_prepare_path_change (BamfMatcher *self, const gchar *desktop_file, ViewChangeType change_type)
 {
   BamfMatcherPrivate *priv;
   BamfApplication *app;
@@ -264,6 +250,12 @@ bamf_matcher_prepare_path_change (BamfMatcher *self, const gchar *desktop_file,
     }
 }
 
+static void
+on_view_closed (BamfView *view, BamfMatcher *self)
+{
+  bamf_matcher_unregister_view (self, view);
+}
+
 static void
 bamf_matcher_register_view_stealing_ref (BamfMatcher *self, BamfView *view)
 {
@@ -275,8 +267,8 @@ bamf_matcher_register_view_stealing_ref (BamfMatcher *self, BamfView *view)
   path = bamf_view_export_on_bus (view, connection);
   type = bamf_view_get_view_type (view);
 
-  g_signal_connect_swapped (G_OBJECT (view), "closed-internal",
-                            (GCallback) bamf_matcher_unregister_view, self);
+  g_signal_connect (G_OBJECT (view), "closed-internal",
+                    (GCallback) on_view_closed, self);
   g_signal_connect (G_OBJECT (view), "active-changed",
                     (GCallback) on_view_active_changed, self);
 
@@ -307,7 +299,7 @@ bamf_matcher_unregister_view (BamfMatcher *self, BamfView *view)
 
   g_signal_emit_by_name (self, "view-closed", path, type);
 
-  g_signal_handlers_disconnect_by_func (G_OBJECT (view), bamf_matcher_unregister_view, self);
+  g_signal_handlers_disconnect_by_func (G_OBJECT (view), on_view_closed, self);
   g_signal_handlers_disconnect_by_func (G_OBJECT (view), on_view_active_changed, self);
 
   if (BAMF_IS_APPLICATION (view))
@@ -359,7 +351,7 @@ get_open_office_window_hint (BamfMatcher * self, BamfLegacyWindow * window)
   else if (g_str_has_suffix (name, "LibreOffice Calc"))
     {
       binary = "libreoffice";
-      parameter = "calc";
+       parameter = "calc";
     }
   else if (g_str_has_suffix (name, "LibreOffice Impress"))
     {
@@ -376,11 +368,6 @@ get_open_office_window_hint (BamfMatcher * self, BamfLegacyWindow * window)
       binary = "libreoffice";
       parameter = "draw";
     }
-  else if (g_str_has_suffix (name, "LibreOffice Base"))
-    {
-      binary = "libreoffice";
-      parameter = "base";
-    }
   else if (g_strcmp0 (class, "libreoffice-startcenter") == 0)
     {
       binary = "libreoffice";
@@ -414,11 +401,6 @@ get_open_office_window_hint (BamfMatcher * self, BamfLegacyWindow * window)
       binary = "ooffice";
       parameter = "draw";
     }
-  else if (g_str_has_suffix (name, "OpenOffice.org Base"))
-    {
-      binary = "ooffice";
-      parameter = "base";
-    }
   else if (g_strcmp0 (name, "OpenOffice.org") == 0 && type == BAMF_WINDOW_NORMAL)
     {
       binary = "ooffice";
@@ -453,11 +435,6 @@ get_open_office_window_hint (BamfMatcher * self, BamfLegacyWindow * window)
             binary = "libreoffice";
             parameter = "draw";
           }
-        else if (g_strcmp0 (class, "libreoffice-base") == 0)
-          {
-            binary = "libreoffice";
-            parameter = "base";
-          }
       }
 
       if (!binary)
@@ -477,72 +454,60 @@ get_open_office_window_hint (BamfMatcher * self, BamfLegacyWindow * window)
   if (!binary)
     return NULL;
 
-  GList *l = NULL;
+  const char *sufix = NULL;
 
-  if (parameter)
+  if (g_strcmp0 (binary, "libreoffice") == 0)
+    sufix = " %U";
+  else if (g_strcmp0 (binary, "ooffice") == 0)
+    sufix = " %F";
+
+  GList *l;
+  if (!parameter)
+    exec = g_strconcat (binary, sufix, NULL);
+  else
+    exec = g_strconcat (binary, " --", parameter, sufix, NULL);
+
+  l = g_hash_table_lookup (self->priv->desktop_file_table, exec);
+  g_free (exec);
+
+  if (!l && parameter)
     {
-      exec = g_strconcat (binary, " --", parameter, NULL);
+      exec = g_strconcat (binary, " -", parameter, sufix, NULL);
+
       l = g_hash_table_lookup (self->priv->desktop_file_table, exec);
       g_free (exec);
-
-      if (!l)
-        {
-          exec = g_strconcat (binary, " -", parameter, NULL);
-          l = g_hash_table_lookup (self->priv->desktop_file_table, exec);
-          g_free (exec);
-
-          if (!l)
-            {
-              exec = g_strconcat (binary, "-", parameter, NULL);
-              l = g_hash_table_lookup (self->priv->desktop_id_table, exec);
-              g_free (exec);
-            }
-        }
-    }
-  else
-    {
-      l = g_hash_table_lookup (self->priv->desktop_file_table, binary);
     }
 
   return (l ? (char *) l->data : NULL);
 }
 
 /* Attempts to return the binary name for a particular execution string */
-char *
-bamf_matcher_get_trimmed_exec (BamfMatcher * self, const char * exec_string)
+static char *
+trim_exec_string (BamfMatcher * self, char * execString)
 {
-  gchar *result = NULL, *part, *tmp;
+  gchar *result = NULL, *exec = NULL, *part = NULL, *tmp = NULL;
   gchar **parts;
-  gint i, j, parts_size;
-  gboolean bad_prefix;
-  gboolean good_prefix = FALSE;
-  gboolean double_parsed = FALSE;
+  gint i, j;
+  gboolean regexFail;
+  gboolean goodPrefix = FALSE;
   GRegex *regex;
 
-  if (!exec_string || exec_string[0] == '\0')
+  if (!execString || (execString && execString[0] == '\0'))
     return NULL;
 
-  if (!g_shell_parse_argv (exec_string, &parts_size, &parts, NULL))
-    return g_strdup (exec_string);
+  exec = g_utf8_casefold (execString, -1);
+  parts = g_strsplit (exec, " ", 0);
 
-  for (i = 0; i < parts_size; ++i)
+  i = 0;
+  while (parts[i] != NULL)
     {
       part = parts[i];
-      if (*part == '%' || g_utf8_strrchr (part, -1, '='))
-        continue;
 
-      if (*part != '-' || good_prefix)
+      if (part[0] != '-')
         {
-          if (!result)
+          if (goodPrefix)
             {
-              tmp = g_utf8_strrchr (part, -1, G_DIR_SEPARATOR);
-              if (tmp)
-                part = tmp + 1;
-            }
-
-          if (good_prefix)
-            {
-              tmp = g_strconcat (result, " ", part, NULL);
+              gchar *tmp = g_strconcat (result, " ", part, NULL);
               g_free (result);
               result = tmp;
             }
@@ -553,80 +518,112 @@ bamf_matcher_get_trimmed_exec (BamfMatcher * self, const char * exec_string)
                   regex = g_array_index (self->priv->good_prefixes, GRegex *, j);
                   if (g_regex_match (regex, part, 0, NULL))
                     {
-                      good_prefix = TRUE;
-                      result = g_ascii_strdown (part, -1);
+                      goodPrefix = TRUE;
+                      result = g_strdup (part);
                       break;
                     }
                 }
+            }
 
-              if (good_prefix)
-                continue;
+          if (!goodPrefix)
+            {
+              tmp = g_utf8_strrchr (part, -1, G_DIR_SEPARATOR);
+              if (tmp)
+                part = tmp + 1;
 
-              bad_prefix = FALSE;
+              regexFail = FALSE;
               for (j = 0; j < self->priv->bad_prefixes->len; j++)
                 {
                   regex = g_array_index (self->priv->bad_prefixes, GRegex *, j);
                   if (g_regex_match (regex, part, 0, NULL))
                     {
-                      bad_prefix = TRUE;
+                      regexFail = TRUE;
                       break;
                     }
                 }
 
-              if (!bad_prefix)
+              if (!regexFail)
                 {
-                  if (!double_parsed && g_utf8_strrchr (part, -1, ' '))
-                  {
-                    /* If the current exec_string has an empty char,
-                     * we double check it again to parse scripts:
-                     * For example strings like 'sh -c "foo || bar"' */
-                    gchar **old_parts = parts;
-
-                    if (g_shell_parse_argv (part, &parts_size, &parts, NULL))
-                      {
-                        // Make the loop to restart!
-                        g_strfreev (old_parts);
-                        i = -1;
-                        continue;
-                      }
-
-                    double_parsed = TRUE;
-                  }
-
-                  result = g_ascii_strdown (part, -1);
+                  result = g_strdup (part);
                   break;
                 }
             }
         }
+      else if (goodPrefix)
+        {
+          break;
+        }
+
+      i++;
     }
 
   if (!result)
     {
-      if (parts_size > 0)
-        {
-          tmp = g_utf8_strrchr (parts[0], -1, G_DIR_SEPARATOR);
-          if (tmp)
-            exec_string = tmp + 1;
-        }
-
-      result = g_strdup (exec_string);
+      result = g_strdup (execString);
     }
   else
     {
       tmp = result;
-
-      regex = g_regex_new ("(\\.bin|\\.py|\\.pl)$", 0, 0, NULL);
+      
+      regex = g_regex_new ("((\\.|-)bin|\\.py)$", 0, 0, NULL);
       result = g_regex_replace_literal (regex, result, -1, 0, "", 0, NULL);
-
+      
       g_free (tmp);
       g_regex_unref (regex);
     }
 
+  g_free (exec);
   g_strfreev (parts);
 
   return result;
 }
 
+static GArray *
+bad_prefix_strings (void)
+{
+  GArray *arr = g_array_new (FALSE, TRUE, sizeof (char *));
+
+  char *str = "^gksu(do)?$";
+  g_array_append_val (arr, str);
+
+  str = "^sudo$";
+  g_array_append_val (arr, str);
+
+  str = "^java$";
+  g_array_append_val (arr, str);
+
+  str = "^mono$";
+  g_array_append_val (arr, str);
+
+  str = "^ruby$";
+  g_array_append_val (arr, str);
+
+  str = "^padsp$";
+  g_array_append_val (arr, str);
+
+  str = "^aoss$";
+  g_array_append_val (arr, str);
+
+  str = "^python(\\d.\\d)?$";
+  g_array_append_val (arr, str);
+
+  str = "^(ba)?sh$";
+  g_array_append_val (arr, str);
+
+  return arr;
+}
+
+static GArray *
+good_prefix_strings (void)
+{
+  GArray *arr = g_array_new (FALSE, TRUE, sizeof (char *));
+
+  char *str = "^gnome-control-center$";
+  g_array_append_val (arr, str);
+
+  return arr;
+}
+
 static GArray *
 pid_parent_tree (BamfMatcher *self, gint pid)
 {
@@ -668,6 +665,15 @@ pid_parent_tree (BamfMatcher *self, gint pid)
   return tree;
 }
 
+static gboolean
+exec_string_should_be_processed (const char *exec)
+{
+  if (!exec)
+    return TRUE;
+
+  return !g_str_has_prefix (exec, "ooffice") && !g_str_has_prefix (exec, "libreoffice");
+}
+
 static gboolean
 is_desktop_folder_item (const char *desktop_file_path, gssize max_len)
 {
@@ -832,15 +838,18 @@ load_desktop_file_to_table (BamfMatcher * self,
       return;
     }
 
-  /**
-   * Set of nasty hacks which should be removed some day. We wish to keep the full exec
-   * strings so we can do nasty matching hacks later. A very very evil thing indeed. However this
-   * helps hack around applications that run in the same process cross radically different instances.
-   * A better solution needs to be thought up, however at this time it is not known.
-   **/
-  char *tmp = bamf_matcher_get_trimmed_exec (self, exec);
-  g_free (exec);
-  exec = tmp;
+  if (exec_string_should_be_processed (exec))
+    {
+      /**
+       * Set of nasty hacks which should be removed some day. We wish to keep the full exec
+       * strings so we can do nasty matching hacks later. A very very evil thing indeed. However this
+       * helps hack around applications that run in the same process cross radically different instances.
+       * A better solution needs to be thought up, however at this time it is not known.
+       **/
+      char *tmp = trim_exec_string (self, exec);
+      g_free (exec);
+      exec = tmp;
+    }
 
   path = g_path_get_basename (file);
   desktop_id = g_string_new (path);
@@ -937,12 +946,16 @@ load_index_file_to_table (BamfMatcher * self,
       GString *desktop_id;
 
       gchar **parts = g_strsplit (line, "\t", 3);
-
-      char *tmp = bamf_matcher_get_trimmed_exec (self, parts[1]);
-      g_free (parts[1]);
-      parts[1] = tmp;
       exec = parts[1];
 
+      if (exec_string_should_be_processed (exec))
+        {
+          char *tmp = trim_exec_string (self, exec);
+          g_free (parts[1]);
+          parts[1] = tmp;
+          exec = parts[1];
+        }
+
       filename = g_build_filename (directory, parts[0], NULL);
 
       desktop_id = g_string_new (parts[0]);
@@ -1147,7 +1160,7 @@ hash_table_remove_sub_values (GHashTable *htable, GCompareFunc compare_func,
                   free_func (l->data);
 
                 /* If the target is the first element of the list (and thanks to
-                 * the previous check we're also sure that it's not the only one),
+-                * the previous check we're also sure that it's not the only one),
                  * simply switch it with its follower, not to change the first
                  * pointer and the hash table value for key
                  */
@@ -1276,25 +1289,15 @@ on_monitor_changed (GFileMonitor *monitor, GFile *file, GFile *other_file, GFile
 }
 
 static void
-bamf_matcher_add_new_monitored_directory (BamfMatcher * self, const gchar *directory)
+bamf_add_new_monitored_directory (BamfMatcher * self, const gchar *directory)
 {
   g_return_if_fail (BAMF_IS_MATCHER (self));
 
   GFile *file;
   GFileMonitor *monitor;
-  GError *error = NULL;
 
   file = g_file_new_for_path (directory);
-  monitor = g_file_monitor_directory (file, G_FILE_MONITOR_NONE, NULL, &error);
-
-  if (error)
-    {
-      g_warning ("Error monitoring %s: %s\n", directory, error->message);
-      g_error_free (error);
-      g_object_unref (file);
-      return;
-    }
-
+  monitor = g_file_monitor_directory (file, G_FILE_MONITOR_NONE, NULL, NULL);
   g_file_monitor_set_rate_limit (monitor, 1000);
   g_object_set_data_full (G_OBJECT (monitor), "root", g_strdup (directory), g_free);
   g_signal_connect (monitor, "changed", (GCallback) on_monitor_changed, self);
@@ -1323,7 +1326,7 @@ fill_desktop_file_table (BamfMatcher * self,
       if (!g_file_test (directory, G_FILE_TEST_IS_DIR))
         continue;
 
-      bamf_matcher_add_new_monitored_directory (self, directory);
+      bamf_add_new_monitored_directory (self, directory);
 
       bamf_file = g_build_filename (directory, "bamf.index", NULL);
 
@@ -1392,6 +1395,24 @@ is_open_office_window (BamfMatcher * self, BamfLegacyWindow * window)
           g_str_has_prefix (class_name, "openoffice"));
 }
 
+static char *
+get_window_hint (BamfLegacyWindow *window, const char *atom_name)
+{
+  g_return_val_if_fail (BAMF_IS_LEGACY_WINDOW (window), NULL);
+
+  Window xid = bamf_legacy_window_get_xid (window);
+  return bamf_xutils_get_window_hint (xid, atom_name, XA_STRING);
+}
+
+static void
+set_window_hint (BamfLegacyWindow * window, const char *atom_name, const char *data)
+{
+  g_return_if_fail (BAMF_LEGACY_WINDOW (window));
+
+  Window xid = bamf_legacy_window_get_xid (window);
+  bamf_xutils_set_window_hint (xid, atom_name, XA_STRING, data);
+}
+
 static char *
 process_exec_string (gint pid)
 {
@@ -1477,7 +1498,8 @@ bamf_matcher_possible_applications_for_pid (BamfMatcher *self,
   
   if (exec_string)
     {
-      trimmed = bamf_matcher_get_trimmed_exec (self, exec_string);
+      trimmed = trim_exec_string (self, exec_string);
+      
       if (trimmed)
         {
           if (trimmed[0] != '\0')
@@ -1631,7 +1653,7 @@ bamf_matcher_possible_applications_for_window (BamfMatcher *self,
 
   priv = self->priv;
   window = bamf_window_get_window (bamf_window);
-  desktop_file = bamf_legacy_window_get_hint (window, _NET_WM_DESKTOP_FILE);
+  desktop_file = get_window_hint (window, _NET_WM_DESKTOP_FILE);
   class_name = bamf_legacy_window_get_class_name (window);
   instance_name = bamf_legacy_window_get_class_instance_name (window);
 
@@ -1866,7 +1888,6 @@ bamf_matcher_get_application_for_window (BamfMatcher *self,
       /* secondary matching */
       GList *a;
       BamfView *view;
-
       const gchar *app_desktop_class;
 
       for (a = self->priv->views; a; a = a->next)
@@ -1877,41 +1898,10 @@ bamf_matcher_get_application_for_window (BamfMatcher *self,
             continue;
 
           app = BAMF_APPLICATION (view);
+          app_desktop_class = bamf_application_get_wmclass (app);
 
           if (bamf_application_contains_similar_to_window (app, bamf_window))
             {
-              char *exec_string = bamf_legacy_window_get_exec_string (window);
-              char *trimmed_exec = bamf_matcher_get_trimmed_exec (self, exec_string);
-              g_free (exec_string);
-
-              GList *ll;
-              gboolean found_exec = FALSE;
-              for (ll = bamf_view_get_children (BAMF_VIEW (app)); ll && !found_exec; ll = ll->next)
-                {
-                  if (!BAMF_IS_WINDOW (ll->data))
-                    continue;
-
-                  BamfLegacyWindow *w = bamf_window_get_window (BAMF_WINDOW (ll->data));
-                  char *wexec = bamf_legacy_window_get_exec_string (w);
-                  char *wtrimmed = bamf_matcher_get_trimmed_exec (self, wexec);
-                  g_free (wexec);
-
-                  if (g_strcmp0 (trimmed_exec, wtrimmed) == 0)
-                    {
-                      best = BAMF_APPLICATION (view);
-                      found_exec = TRUE;
-                    }
-
-                  g_free (wtrimmed);
-                }
-
-              g_free (trimmed_exec);
-
-              if (!found_exec)
-                continue;
-
-              app_desktop_class = bamf_application_get_wmclass (app);
-
               if (target_class && g_strcmp0 (target_class, app_desktop_class) == 0)
                 {
                   best = app;
@@ -1980,7 +1970,7 @@ ensure_window_hint_set (BamfMatcher *self,
       return;
     }
 
-  desktop_file_hint = bamf_legacy_window_get_hint (window, _NET_WM_DESKTOP_FILE);
+  desktop_file_hint = get_window_hint (window, _NET_WM_DESKTOP_FILE);
 
   if (desktop_file_hint)
     {
@@ -2017,7 +2007,7 @@ ensure_window_hint_set (BamfMatcher *self,
   g_array_free (pids, TRUE);
 
   if (desktop_file_hint)
-    bamf_legacy_window_set_hint (window, _NET_WM_DESKTOP_FILE, desktop_file_hint);
+    set_window_hint (window, _NET_WM_DESKTOP_FILE, desktop_file_hint);
 }
 
 static void
@@ -2062,7 +2052,7 @@ on_open_office_window_name_changed (BamfLegacyWindow *window, BamfMatcher* self)
   char *old_hint;
   const char *new_hint;
 
-  old_hint = bamf_legacy_window_get_hint (window, _NET_WM_DESKTOP_FILE);
+  old_hint = get_window_hint (window, _NET_WM_DESKTOP_FILE);
   new_hint = get_open_office_window_hint (self, window);
 
   if (new_hint && g_strcmp0 (new_hint, old_hint) != 0)
@@ -2084,26 +2074,26 @@ static char *
 get_gnome_control_center_window_hint (BamfMatcher * self, BamfLegacyWindow * window)
 {
   gchar *role;
+  gchar *exec;
+  GHashTable *desktopFileTable;
   GList *list;
 
   g_return_val_if_fail (BAMF_IS_MATCHER (self), NULL);
   g_return_val_if_fail (BAMF_IS_LEGACY_WINDOW (window), NULL);
 
-  list = NULL;
-  role = bamf_legacy_window_get_hint (window, WM_WINDOW_ROLE);
+  role = get_window_hint (window, WM_WINDOW_ROLE);
+  exec = g_strconcat ("gnome-control-center", (role ? " " : NULL), role, NULL);
 
-  if (role)
+  desktopFileTable = self->priv->desktop_file_table;
+  list = g_hash_table_lookup (desktopFileTable, exec);
+
+  if (!list && g_strcmp0 ("gnome-control-center", exec) != 0)
     {
-      gchar *exec = g_strconcat ("gnome-control-center ", role, NULL);
-      list = g_hash_table_lookup (self->priv->desktop_file_table, exec);
-      g_free (exec);
-      g_free (role);
+      list = g_hash_table_lookup (desktopFileTable, "gnome-control-center");
     }
 
-  if (!role || !list)
-    {
-      list = g_hash_table_lookup (self->priv->desktop_id_table, "gnome-control-center");
-    }
+  g_free (exec);
+  g_free (role);
 
   return (list ? (char *) list->data : NULL);
 }
@@ -2117,7 +2107,7 @@ on_gnome_control_center_window_name_changed (BamfLegacyWindow *window, BamfMatch
   char *old_hint;
   const char *new_hint;
 
-  old_hint = bamf_legacy_window_get_hint (window, _NET_WM_DESKTOP_FILE);
+  old_hint = get_window_hint (window, _NET_WM_DESKTOP_FILE);
   new_hint = get_gnome_control_center_window_hint (self, window);
 
   if (new_hint && g_strcmp0 (new_hint, old_hint) != 0)
@@ -2141,8 +2131,7 @@ handle_window_opened (BamfLegacyScreen * screen, BamfLegacyWindow * window, Bamf
   g_return_if_fail (BAMF_IS_MATCHER (self));
   g_return_if_fail (BAMF_IS_LEGACY_WINDOW (window));
 
-  BamfWindowType win_type = bamf_legacy_window_get_window_type (window);
-  if (win_type == BAMF_WINDOW_DESKTOP)
+  if (bamf_legacy_window_get_window_type (window) == BAMF_WINDOW_DESKTOP)
     {
       BamfWindow *bamfwindow = bamf_window_new (window);
       bamf_matcher_register_view_stealing_ref (self, BAMF_VIEW (bamfwindow));
@@ -2152,17 +2141,19 @@ handle_window_opened (BamfLegacyScreen * screen, BamfLegacyWindow * window, Bamf
 
   if (is_open_office_window (self, window))
     {
+      BamfWindowType win_type = bamf_legacy_window_get_window_type (window);
+
       if (win_type == BAMF_WINDOW_SPLASHSCREEN || win_type == BAMF_WINDOW_TOOLBAR)
         {
           return;
         }
 
-      char *old_hint = bamf_legacy_window_get_hint (window, _NET_WM_DESKTOP_FILE);
+      char *old_hint = get_window_hint (window, _NET_WM_DESKTOP_FILE);
       const char *new_hint = get_open_office_window_hint (self, window);
 
       if (new_hint && g_strcmp0 (old_hint, new_hint) != 0)
         {
-          bamf_legacy_window_set_hint (window, _NET_WM_DESKTOP_FILE, new_hint);
+          set_window_hint (window, _NET_WM_DESKTOP_FILE, new_hint);
         }
 
       g_object_ref (window);
@@ -2173,12 +2164,12 @@ handle_window_opened (BamfLegacyScreen * screen, BamfLegacyWindow * window, Bamf
     }
   else if (g_strcmp0 (bamf_legacy_window_get_class_name (window), "Gnome-control-center") == 0)
     {
-      char *old_hint = bamf_legacy_window_get_hint (window, _NET_WM_DESKTOP_FILE);
+      char *old_hint = get_window_hint (window, _NET_WM_DESKTOP_FILE);
       const char *new_hint = get_gnome_control_center_window_hint (self, window);
 
       if (new_hint && g_strcmp0 (old_hint, new_hint) != 0)
         {
-          bamf_legacy_window_set_hint (window, _NET_WM_DESKTOP_FILE, new_hint);
+          set_window_hint (window, _NET_WM_DESKTOP_FILE, new_hint);
         }
 
       g_object_ref (window);
@@ -2904,35 +2895,67 @@ on_dbus_handle_window_stack_for_monitor (BamfDBusMatcher *interface,
   return TRUE;
 }
 
+static void
+on_webapp_child_added (BamfView *application,
+                       BamfView *child,
+                       gpointer user_data)
+{
+  BamfMatcher *self;
+  
+  self = (BamfMatcher *)user_data;
+  bamf_matcher_register_view_stealing_ref (self, child);
+}
+
+static void
+on_webapp_appeared (BamfUnityWebappsObserver *observer,
+                    BamfApplication *application,
+                    gpointer user_data)
+{
+  BamfMatcher *self;
+  
+  self = (BamfMatcher *)user_data;
+  
+  bamf_matcher_register_view_stealing_ref (self, (BamfView *)application);
+  
+  g_signal_connect (application, "tab-appeared", G_CALLBACK (on_webapp_child_added),
+                    self);
+  bamf_unity_webapps_application_add_existing_interests (BAMF_UNITY_WEBAPPS_APPLICATION (application));
+}
+
 static void
 bamf_matcher_init (BamfMatcher * self)
 {
   BamfMatcherPrivate *priv;
   BamfLegacyScreen *screen;
   BamfIndicatorSource *approver;
+  GArray *prefixstrings;
   int i;
 
   priv = self->priv = BAMF_MATCHER_GET_PRIVATE (self);
 
   priv->known_pids = g_array_new (FALSE, TRUE, sizeof (gint));
-  priv->bad_prefixes = g_array_sized_new (FALSE, TRUE, sizeof (GRegex *),
-                                          G_N_ELEMENTS (EXEC_BAD_PREFIXES));
-  priv->good_prefixes = g_array_sized_new (FALSE, TRUE, sizeof (GRegex *),
-                                           G_N_ELEMENTS (EXEC_GOOD_PREFIXES));
+  priv->bad_prefixes = g_array_new (FALSE, TRUE, sizeof (GRegex *));
+  priv->good_prefixes = g_array_new (FALSE, TRUE, sizeof (GRegex *));
   priv->registered_pids = g_hash_table_new_full (g_direct_hash, g_direct_equal,
                                                  NULL, g_free);
 
-  for (i = 0; i < G_N_ELEMENTS (EXEC_BAD_PREFIXES); ++i)
+  prefixstrings = bad_prefix_strings ();
+  for (i = 0; i < prefixstrings->len; i++)
     {
-      GRegex *regex = g_regex_new (EXEC_BAD_PREFIXES[i], G_REGEX_OPTIMIZE, 0, NULL);
+      char *str = g_array_index (prefixstrings, char *, i);
+      GRegex *regex = g_regex_new (str, G_REGEX_OPTIMIZE, 0, NULL);
       g_array_append_val (priv->bad_prefixes, regex);
     }
+  g_array_free (prefixstrings, TRUE);
 
-  for (i = 0; i < G_N_ELEMENTS (EXEC_GOOD_PREFIXES); ++i)
+  prefixstrings = good_prefix_strings ();
+  for (i = 0; i < prefixstrings->len; i++)
     {
-      GRegex *regex = g_regex_new (EXEC_GOOD_PREFIXES[i], G_REGEX_OPTIMIZE, 0, NULL);
+      char *str = g_array_index (prefixstrings, char *, i);
+      GRegex *regex = g_regex_new (str, G_REGEX_OPTIMIZE, 0, NULL);
       g_array_append_val (priv->good_prefixes, regex);
     }
+  g_array_free (prefixstrings, TRUE);
 
   create_desktop_file_table (self, &(priv->desktop_file_table),
                              &(priv->desktop_id_table),
@@ -2990,6 +3013,13 @@ bamf_matcher_init (BamfMatcher * self)
 
   g_signal_connect (self, "handle-window-stack-for-monitor",
                     G_CALLBACK (on_dbus_handle_window_stack_for_monitor), self);
+  
+  priv->webapps_observer = bamf_unity_webapps_observer_new ();
+  
+  g_signal_connect (priv->webapps_observer,
+                    "application-appeared",
+                    G_CALLBACK (on_webapp_appeared),
+                    self);
 }
 
 static void
@@ -3002,6 +3032,12 @@ bamf_matcher_dispose (GObject *object)
     {
       bamf_matcher_unregister_view (self, priv->views->data);
     }
+  
+  if (priv->webapps_observer)
+    {
+      g_object_unref (G_OBJECT (priv->webapps_observer));
+      priv->webapps_observer = NULL;
+    }
 
   G_OBJECT_CLASS (bamf_matcher_parent_class)->dispose (object);
 }
@@ -3056,7 +3092,6 @@ bamf_matcher_finalize (GObject *object)
       g_signal_handlers_disconnect_by_func (G_OBJECT (l->data), 
                                             (GCallback) on_monitor_changed, self);
     }
-
   g_list_free_full (priv->monitors, g_object_unref);
   priv->monitors = NULL;
 
